/// Print a message to stderr and exit with status code 1 to avoid using 
/// .expect() or .unwrap()
///
/// Arguments are passed to `format!` to format the message
#[macro_export]
macro_rules! fatal {
    ($($arg:tt)*) => {
        {
            eprintln!("[FATAL] {}", format!($($arg)*));
            std::process::exit(1);
        }
    }
}

pub trait UnwrapOrFatalAble<T> {
    fn unwrap_or_fatal(self, msg: &str) -> T;
}

impl<T> UnwrapOrFatalAble<T> for Option<T> {
    /// Unwraps an `Option<T>` into its inner value or exits the program with
    /// a fatal message
    fn unwrap_or_fatal(self, msg: &str) -> T {
        match self {
            Some(val) => val,
            None => fatal!("{}", msg),
        }
    }
}

impl<T, E> UnwrapOrFatalAble<T> for Result<T, E>
where
    E: std::fmt::Display,
{
    /// Unwraps a `Result<T, E>` into its inner value or exits the program with  
    /// a fatal message
    fn unwrap_or_fatal(self, msg: &str) -> T {
        match self {
            Ok(val) => val,
            Err(e) => fatal!("{}: {}", msg, e),
        }
    }
}
