use diaryrs::{macros::UnwrapOrFatalAble, time::Time};
use regex::Regex;

use crate::args::Config;

use std::{fs, path};

// Management for existing diary entries

/// Overall stats and information about existing diary entries
///
/// Call `::take` to construct an `Inventory` struct from diary entires on disk
#[derive(Debug)]
pub struct Inventory {
    /// Number of entries in the diary
    pub entries: usize,

    /// Total word count of all entries
    ///
    /// Does not include YAML front matter or comments
    pub word_count: u64,

    /// Average word count: simply `word_count / entries` rounded to the nearest integer
    pub avg_word_count: f64,
}

impl Inventory {
    /// Take inventory of the diary entries
    ///
    /// Reads the diary entries from disks and produces a useful
    /// `Inventory` struct with stats and information about the entries.
    pub fn take(config: &Config) -> Result<Self, Box<dyn std::error::Error>> {
        // Take directory listing
        let dir = fs::read_dir(&config.base_path)?;

        let md_paths: Vec<_> = dir
            .filter_map(|entry| entry.ok())
            .flat_map(|entry| recurse_paths_md(entry))
            .flatten()
            .collect();

        let word_count = word_count_mds(&md_paths)?;
        let entries_count = md_paths.len();

        Ok(Self {
            entries: entries_count,
            word_count,
            avg_word_count: (word_count as f64 / entries_count as f64).round(),
        })
    }
}

/// Recursively finds every Markdown (*.md) file in a directory
/// and its subdirectories
///
/// Returns a vector of `PathBuf`s to each Markdown file or a boxed `Error`.
fn recurse_paths_md(dir: fs::DirEntry) -> Result<Vec<path::PathBuf>, Box<dyn std::error::Error>> {
    let mut paths = Vec::new();
    let current_path = dir.path();
    let path_type = dir.file_type()?;

    if path_type.is_dir() {
        // List dir, then run this function on each entry
        // Add anything returned to the paths vector
        let dir = fs::read_dir(&current_path)?;
        paths.extend(
            dir.filter_map(|entry| entry.ok())
                .flat_map(|entry: fs::DirEntry| recurse_paths_md(entry))
                .flatten(),
        );
    }

    if path_type.is_file() {
        // Check if file is an md file
        if let Some(ext) = current_path.extension() {
            if ext == "md" {
                paths.push(current_path);
            }
        }
    }

    Ok(paths)
}

fn word_count_mds(paths: &Vec<path::PathBuf>) -> Result<u64, Box<dyn std::error::Error>> {
    // temp regexes
    let re_titles = Regex::new(r"(?m)^#{1,6} .+")
        .unwrap_or_fatal("Failed to compile title regex. Something is very wrong!");
    let re_comments = Regex::new(r"(?s-m)<!---?.+-->")
        .unwrap_or_fatal("Failed to compile comment regex. Something is very wrong!");
    let re_images = Regex::new(r"!\[.*\]\(.+\)")
        .unwrap_or_fatal("Failed to compile image regex. Something is very wrong!");

    let mut word_count = 0;

    for path in paths {
        let contents = fs::read_to_string(&path)?;

        // Cut YAML header and comments
        let contents = &re_titles.replace_all(
            contents
                .split("---")
                .collect::<Vec<&str>>()
                .pop()
                .ok_or("No content found")?,
            "",
        );
        let contents = &re_comments.replace_all(contents, "");
        let contents = &re_images.replace_all(contents, "");

        word_count += contents.split_whitespace().count();
    }

    Ok(word_count as u64)
}
